/*
Try to make an external VFO fot the TS-520S.
Display 000Khz, VFO = 5.5Mhz
Display 600Khz, VFO = 4.9Hhz

Code for LCD not finished as yet so use an OLED display.

Some stuff is not as yet implemented but the basic VFO is working.
*/

#define Version "TS-520S_20240418"

// comment out line below for LCD
#define USE_OLED

// a handy way to swap the encoder dorection without rewiring
#define SwapEncoder

#include "si5351.h"
#include "Wire.h"

// synthesiser default settings on start up.
unsigned long frequency =  5500000;     // 4 bytes required for frequency
unsigned long step_size =     1000;     // default tuning increment (Hz)
         long RIT_freq  =        0;     // RIT offset for clockgen
unsigned long VFO       =        0;     // VFO frequency

#define       RX                HIGH
int           TRXmode         = RX;          // HIGH = RX, LOW = TX

int32_t cal_factor = 33100; //  for my radio, found from Si5351 calibrator sketch.
byte updateFlag=HIGH;
int  temp = 0;              // temperary scratch pad
int loops = 0;
long encoder;                               // encoder direction: CCW=-1; NC=0; CW=1
Si5351 clockgen;                            // initalize clockgen

// port definitions
#define ENCODER_A  A0
#define ENCODER_B  A1
#define TRX        A2                      // 
#define ModeSelect A3                      // AM, USB, LSB 
// A4 = SDA
// A5 = SCL
#define RITpot     A6                  // RIT input
#define AnalogIn   A7                  // not used yet

#define Input1   2                        // not used yet
#define Input2   3                        // not used yet
#define Output2  4                        // not used yet
#define ENCODER_BUTTON 5                  // selects tuning decade
#define MuteIn    6                       // 
#define Output1  13                       // not used yet
                           
#define ENCODER_INPUT_PORT PINC
int sideband = 0;   // 0 = AM, 1 = USB, 2 = LSB
int last_sideband = 0;
long RIT = 0;
long lastRIT = 0;

// Code for LCD not finished as yet so use an OLED display.
#ifdef USE_OLED
#include "oled_ui.h"
#else
#include "lcd_ui.h"
#endif

/* HF Ham bands
Band        UK Allocation          USA Allocation    Australia Allocation
(metres)          Mhz                   Mhz                 Mhz
 160         1.810 -  2.000        1.800 -  2.000        1.800 -  1.875
  80         3.500 -  3.800        3.500 -  4.000        3.500 -  3.800
  40         7.000 -  7.200        7.000 -  7.300        7.000 -  7.300
  30        10.100 - 10.150       10.100 - 10.150       10.100 - 10.150
  20        14.100 - 14.350       14.100 - 14.350       14.100 - 14.350
  17        18.068 - 18.168       18.068 - 18.168       18.068 - 18.168
  15        21.000 - 21.450       21.000 - 21.450       21.000 - 21.450 
  12        24.890 - 24.990       24.890 - 24.990       24.890 - 24.990
  10        28.000 - 29.700       28.000 - 29.700       28.000 - 29.700 
 */
 
void setup() { 

  setup_display();

  // Start serial and initialize the Si5351
  Serial.begin(57600);
  clockgen.init(SI5351_CRYSTAL_LOAD_8PF, 0, 0);
  clockgen.set_correction(cal_factor, SI5351_PLL_INPUT_XO); // <<<<<<<this is found from Si5351 calibrator code.

  // configure rotary encoder & push-button
  pinMode(ENCODER_A, INPUT_PULLUP);     // encoder leads
  pinMode(ENCODER_B, INPUT_PULLUP);  
  pinMode(ENCODER_BUTTON, INPUT_PULLUP);
  pinMode(TRX, INPUT);            // 
  
  display_freq_step();            // display frequency & step-size
  set_synth();
}

/******************************
 main loop
*******************************/
void loop() {

// check the RIT

loops ++;
if (loops >50)                        // every 50th loop, check the RIT...
 { RIT = (analogRead(RITpot));        // try to filter the noise out....
  if ( RIT==(analogRead(RITpot)))     // read twice the same in a row...
   {if ( RIT==(analogRead(RITpot)))   // read thrice the same in a row...
     { loops = 0;
       {if (RIT!=lastRIT)
         {display.setCursor(50,0);
          display.print("RIT=     ");
          if (RIT <= 10)
           { RIT_freq = 0;
           }
        else
           {lastRIT = RIT;
            RIT_freq = (RIT-400);
            if (RIT_freq >=2) 
             { RIT_freq = RIT_freq/2;
             }
            display.setCursor(80,0);
            display.print(RIT_freq);
           }
            display.display(); // Update screen
         }
       }
     }
   }
 }

  // check the frequency......
  check_step_size();
  encoder = read_encoder();                       // returns +1/0/-1 (cw/nc/ccw)
  if (encoder != 0) {
    frequency = frequency + (encoder * step_size); 
    frequency = constrain(frequency, 4900000, 5500000);
    set_synth();
    updateFlag=HIGH;
  } 
 
  if     (updateFlag==HIGH)
  {    display_freq_step();                       // display frequency & step-size
  }
}

/******************************
 read encoder direction
*******************************/
char read_encoder() {
  // definitions
  #define DELAY 100                               // number of polling-loops   
  static unsigned short integrator;               // encoder debounce
 
  /* the array values are determined by combining the previous and current A1,A0 
  Gray Code bit-patterns to form an array-index. +1 is assigned to all clockwise 
  (CW) indexes; -1 is assigned to all counter-clockwise (CCW) indexes; and 0 is 
  assigned to the remaining positions to indicate no movement*/   
  static char encoder[] = {0,-1,1,0,1,0,0,-1,-1,0,0,1,0,1,-1,0}; // SWAPPED PHASES
  //   static char encoder[] = {0,1,-1,0,-1,0,0,1,1,0,0,-1,0,-1,1,0};
  static unsigned char encoder_index = 0;
  
  static long count = 0;                         // CW rotation = +1; CCW rotation = -1
  static long this_reading;
  static long last_reading;
  char value;                                    // return value

  /* check for encoder rotation. The absolute "count" value changes as the encoder rotates.
  Typically there are four states (counts) per click, but often more due to contact bounce.
  Bounce doesn't matter so long as the "count" decreases" for CCW rotation; remains constant 
  when there is no rotation; and increases with CW rotation. Bounce can be completely 
  eliminated if we look for the "detent" code and integrate as shown below. */  
  encoder_index <<= 2;                           // shift previous bit-pattern left two places
  encoder_index |= (ENCODER_INPUT_PORT & 0x03);  // get current bit-pattern and combine with previous
  count += encoder[( encoder_index & 0x0f )];     
  
  // debounce encoder
  if ((ENCODER_INPUT_PORT & 0x03) == 3) {        // encoder on an indent (3 decimal == 00000011 binary)
    if (integrator < DELAY) {                    
      integrator++;
    } 
  } else {                                       // encoder not resting on indent
     if (integrator > 0) {
       integrator--;
     }
  }
  
  // prepare the "return" value
  if (integrator >= DELAY) {                     // encoder deemed to be stationary
    this_reading = count;

#ifdef SwapEncoder
    if (this_reading < last_reading) value =  1;
    if (this_reading == last_reading) value = 0;
    if (this_reading > last_reading) value = -1;
#else
    if (this_reading < last_reading) value = -1;
    if (this_reading == last_reading) value = 0;
    if (this_reading > last_reading) value =  1;
#endif
   
    last_reading = this_reading;                 // both readings now hold current "count"
    integrator = 0;                              // reset integrator
  } else {
    value = 0;
  }
  return(value);
}
  



/******************************
 check the step size
*******************************/
void check_step_size() {
  // delay definitions
  #define DELAY 30                             // DELAY = debounce(mS)*loops/second
  #define LONG_PUSH 50                        // number of loops considered "long"
  
  // variables
  static boolean transition = false;             // valid during button push
  static unsigned short integrator;              // varies from 0 to MAXIMUM
  static unsigned short push_duration;           // determines step size/direction
  
  // look for first high-to-low transition
  if (!transition && (digitalRead(ENCODER_BUTTON) == LOW)) {
    transition = true;
    integrator = 0;
    push_duration = 0;
  }
  
  // look for last low-to-high transition
  if (transition) {
    if (push_duration < (LONG_PUSH*2)) {         // track push duration
      push_duration++;
    }
    if (digitalRead(ENCODER_BUTTON) == HIGH) {   // ENCODER_BUTTON is HIGH
      if (integrator < DELAY) {        
        integrator++;
      } else {                                   // ENCODER_BUTTON is LOW
        if (integrator > 0) {
    integrator--;
  }
      }
    }
   
    // update the step_size
    if (integrator >= DELAY) { 
      transition = false;
      if (push_duration > LONG_PUSH) {          // LONG PUSH: decrease step size
        step_size = step_size/10;
        if (step_size < 10) {                    //
          step_size = 100000;                   // wrap around    
        }        
      } else {                                  // SHORT PUSH: increase step size
        step_size = step_size*10;
        if (step_size > 100000) {
          step_size = 10;                       // wrap around 
        }         
      }
      
      display_freq_step();                      // update step-size     
    }
  }
}


/*******************************
set the Si5351 CLK0 frequency
 ******************************/
void set_synth ()
{ VFO = (frequency +RIT_freq);
  clockgen.set_freq((VFO*100), SI5351_CLK0); // "VFO"

}
 
